@@ -48,16 +48,17 @@ static const GtkTargetEntry tb_editor_dnd_targets[] =
 };
 static const gint tb_editor_dnd_targets_len = G_N_ELEMENTS(tb_editor_dnd_targets);
 
+static void tb_editor_browser_populate_tool_menu_cb(MidoriBrowser *browser, GtkWidget *menu, MidoriExtension *ext);
 
 static void tb_editor_app_add_browser_cb(MidoriApp *app, MidoriBrowser *browser, MidoriExtension *ext);
 
 
-static void tb_editor_deactivate_cb(MidoriExtension *extension, GtkWidget *menuitem)
+static void tb_editor_deactivate_cb(MidoriExtension *extension, MidoriBrowser *browser)
 {
 	MidoriApp *app = midori_extension_get_app(extension);
 
-	gtk_widget_destroy(menuitem);
-	g_signal_handlers_disconnect_by_func(extension, tb_editor_deactivate_cb, menuitem);
+	g_signal_handlers_disconnect_by_func(browser, tb_editor_browser_populate_tool_menu_cb, extension);
+	g_signal_handlers_disconnect_by_func(extension, tb_editor_deactivate_cb, browser);
 	g_signal_handlers_disconnect_by_func(app, tb_editor_app_add_browser_cb, extension);
 }
 
@@ -571,24 +572,21 @@ static void tb_editor_menu_configure_toolbar_activate_cb(GtkWidget *menuitem, Mi
 	g_free(tbw);
 }
 
+static void tb_editor_browser_populate_tool_menu_cb(MidoriBrowser *browser, GtkWidget *menu, MidoriExtension *ext)
+{
+    GtkWidget *menuitem;
+
+    menuitem = gtk_menu_item_new_with_mnemonic (_("Customize _Toolbar..."));
+    g_signal_connect (menuitem, "activate",
+        G_CALLBACK (tb_editor_menu_configure_toolbar_activate_cb), browser);
+    gtk_widget_show (menuitem);
+    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+}
 
 static void tb_editor_app_add_browser_cb(MidoriApp *app, MidoriBrowser *browser, MidoriExtension *ext)
 {
-	GtkWidget *panel;
-	GtkWidget *menu;
-	GtkWidget *menuitem;
-
-	panel = katze_object_get_object(browser, "panel");
-	menu = katze_object_get_object(panel, "menu");
-	g_object_unref(panel);
-	menuitem = gtk_menu_item_new_with_mnemonic(_("Configure _Toolbar..."));
-	g_signal_connect(menuitem, "activate",
-		G_CALLBACK(tb_editor_menu_configure_toolbar_activate_cb), browser);
-	gtk_widget_show(menuitem);
-	gtk_menu_shell_insert(GTK_MENU_SHELL (menu), menuitem, 3);
-	g_object_unref(menu);
-
-	g_signal_connect(ext, "deactivate", G_CALLBACK(tb_editor_deactivate_cb), menuitem);
+	g_signal_connect(browser, "populate-tool-menu", G_CALLBACK(tb_editor_browser_populate_tool_menu_cb), ext);
+	g_signal_connect(ext, "deactivate", G_CALLBACK(tb_editor_deactivate_cb), browser);
 }
 
 
@@ -606,20 +604,22 @@ static void tb_editor_activate_cb(MidoriExtension *extension, MidoriApp *app)
 	g_object_unref(browsers);
 }
 
+#endif
 
 MidoriExtension *extension_init(void)
 {
 	MidoriExtension* extension = g_object_new(MIDORI_TYPE_EXTENSION,
 		"name", _("Toolbar Editor"),
 		"description", _("Easily edit the toolbar layout"),
+		#if !HAVE_HILDON
 		"version", "0.1",
+		#endif
 		"authors", "Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>",
 		NULL);
 
+	#if !HAVE_HILDON
 	g_signal_connect(extension, "activate", G_CALLBACK(tb_editor_activate_cb), NULL);
+	#endif
 
 	return extension;
 }
-
-
-#endif
