@@ -116,6 +116,9 @@ midori_view_get_proxy_menu_item        (MidoriView*        view);
 GtkWidget*
 midori_view_get_tab_menu               (MidoriView*        view);
 
+PangoEllipsizeMode
+midori_view_get_label_ellipsize        (MidoriView*        view);
+
 GtkWidget*
 midori_view_get_proxy_tab_label        (MidoriView*        view);
 
