@@ -48,6 +48,7 @@ struct _MidoriWebSettings
     gboolean search_engines_in_completion;
     gchar* toolbar_items;
     gboolean compact_sidepanel;
+    gboolean show_panel_controls;
     gboolean right_align_sidepanel;
 
     MidoriStartup load_on_startup;
@@ -123,6 +124,7 @@ enum
     PROP_SEARCH_ENGINES_IN_COMPLETION,
     PROP_TOOLBAR_ITEMS,
     PROP_COMPACT_SIDEPANEL,
+    PROP_SHOW_PANEL_CONTROLS,
     PROP_RIGHT_ALIGN_SIDEPANEL,
 
     PROP_LOAD_ON_STARTUP,
@@ -192,9 +194,9 @@ midori_startup_get_type (void)
     if (!type)
     {
         static const GEnumValue values[] = {
-         { MIDORI_STARTUP_BLANK_PAGE, "MIDORI_STARTUP_BLANK_PAGE", N_("Blank page") },
-         { MIDORI_STARTUP_HOMEPAGE, "MIDORI_STARTUP_HOMEPAGE", N_("Homepage") },
-         { MIDORI_STARTUP_LAST_OPEN_PAGES, "MIDORI_STARTUP_LAST_OPEN_PAGES", N_("Last open pages") },
+         { MIDORI_STARTUP_BLANK_PAGE, "MIDORI_STARTUP_BLANK_PAGE", N_("Show Blank page") },
+         { MIDORI_STARTUP_HOMEPAGE, "MIDORI_STARTUP_HOMEPAGE", N_("Show Homepage") },
+         { MIDORI_STARTUP_LAST_OPEN_PAGES, "MIDORI_STARTUP_LAST_OPEN_PAGES", N_("Show last open tabs") },
          { 0, NULL, NULL }
         };
         type = g_enum_register_static ("MidoriStartup", values);
@@ -249,8 +251,8 @@ midori_toolbar_style_get_type (void)
          { MIDORI_TOOLBAR_DEFAULT, "MIDORI_TOOLBAR_DEFAULT", N_("Default") },
          { MIDORI_TOOLBAR_ICONS, "MIDORI_TOOLBAR_ICONS", N_("Icons") },
          { MIDORI_TOOLBAR_TEXT, "MIDORI_TOOLBAR_TEXT", N_("Text") },
-         { MIDORI_TOOLBAR_BOTH, "MIDORI_TOOLBAR_BOTH", N_("Both") },
-         { MIDORI_TOOLBAR_BOTH_HORIZ, "MIDORI_TOOLBAR_BOTH_HORIZ", N_("Both horizontal") },
+         { MIDORI_TOOLBAR_BOTH, "MIDORI_TOOLBAR_BOTH", N_("Icons and text") },
+         { MIDORI_TOOLBAR_BOTH_HORIZ, "MIDORI_TOOLBAR_BOTH_HORIZ", N_("Text beside icons") },
          { 0, NULL, NULL }
         };
         type = g_enum_register_static ("MidoriToolbarStyle", values);
@@ -473,7 +475,7 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      PROP_TOOLBAR_STYLE,
                                      g_param_spec_enum (
                                      "toolbar-style",
-                                     _("Toolbar Style"),
+                                     _("Toolbar Style:"),
                                      _("The style of the toolbar"),
                                      MIDORI_TYPE_TOOLBAR_STYLE,
                                      MIDORI_TOOLBAR_DEFAULT,
@@ -529,6 +531,22 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      FALSE,
                                      flags));
 
+    /**
+     * MidoriWebSettings:show-panel-controls:
+     *
+     * Whether to show the operating controls of the panel.
+     *
+     * Since: 0.1.9
+     */
+    g_object_class_install_property (gobject_class,
+                                     PROP_SHOW_PANEL_CONTROLS,
+                                     g_param_spec_boolean (
+                                     "show-panel-controls",
+                                     _("Show operating controls of the panel"),
+                                     _("Whether to show the operating controls of the panel"),
+                                     TRUE,
+                                     flags));
+
     /**
     * MidoriWebSettings:right-sidepanel:
     *
@@ -550,8 +568,8 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      PROP_LOAD_ON_STARTUP,
                                      g_param_spec_enum (
                                      "load-on-startup",
-                                     _("Load on Startup"),
-                                     _("What to load on startup"),
+                                     _("When Midori starts:"),
+                                     _("What to do when Midori starts"),
                                      MIDORI_TYPE_STARTUP,
                                      MIDORI_STARTUP_LAST_OPEN_PAGES,
                                      flags));
@@ -560,7 +578,7 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      PROP_HOMEPAGE,
                                      g_param_spec_string (
                                      "homepage",
-                                     _("Homepage"),
+                                     _("Homepage:"),
                                      _("The homepage"),
                                      "http://www.google.com",
                                      flags));
@@ -602,7 +620,7 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      PROP_DOWNLOAD_FOLDER,
                                      g_param_spec_string (
                                      "download-folder",
-                                     _("Download Folder"),
+                                     _("Save downloaded files to:"),
                                      _("The folder downloaded files are saved to"),
                                      midori_get_download_dir (),
     #if WEBKIT_CHECK_VERSION (1, 1, 3)
@@ -727,7 +745,7 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      PROP_OPEN_NEW_PAGES_IN,
                                      g_param_spec_enum (
                                      "open-new-pages-in",
-                                     _("Open new pages in"),
+                                     _("Open new pages in:"),
                                      _("Where to open new pages"),
                                      MIDORI_TYPE_NEW_PAGE,
                                      MIDORI_NEW_PAGE_TAB,
@@ -737,7 +755,7 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      PROP_OPEN_EXTERNAL_PAGES_IN,
                                      g_param_spec_enum (
                                      "open-external-pages-in",
-                                     _("Open external pages in"),
+                                     _("Open external pages in:"),
                                      _("Where to open externally opened pages"),
                                      MIDORI_TYPE_NEW_PAGE,
                                      MIDORI_NEW_PAGE_TAB,
@@ -749,7 +767,7 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      "middle-click-opens-selection",
                                      _("Middle click opens Selection"),
                                      _("Load an address from the selection via middle click"),
-                                     FALSE,
+                                     TRUE,
                                      flags));
 
     g_object_class_install_property (gobject_class,
@@ -1000,6 +1018,7 @@ midori_web_settings_init (MidoriWebSettings* web_settings)
     web_settings->notify_transfer_completed = TRUE;
     web_settings->download_folder = g_strdup (midori_get_download_dir ());
     web_settings->http_proxy = NULL;
+    web_settings->show_panel_controls = TRUE;
     web_settings->open_popups_in_tabs = TRUE;
     web_settings->remember_last_form_inputs = TRUE;
     web_settings->remember_last_downloaded_files = TRUE;
@@ -1086,15 +1105,23 @@ generate_ident_string (MidoriIdentity identify_as)
 
     const gchar* lang = pango_language_to_string (gtk_get_default_language ());
 
+    #ifndef WEBKIT_USER_AGENT_MAJOR_VERSION
+        #define WEBKIT_USER_AGENT_MAJOR_VERSION 532
+        #define WEBKIT_USER_AGENT_MINOR_VERSION 1
+    #endif
+
+    const gchar* webcore = "WebKit/" G_STRINGIFY (WEBKIT_USER_AGENT_MAJOR_VERSION)
+        "." G_STRINGIFY (WEBKIT_USER_AGENT_MINOR_VERSION) "+";
+
     switch (identify_as)
     {
     case MIDORI_IDENT_MIDORI:
-        return g_strdup_printf ("%s (%s; %s; U; %s) WebKit/532+",
-                                appname, platform, os, lang);
+        return g_strdup_printf ("%s (%s; %s; U; %s) %s",
+                                appname, platform, os, lang, webcore);
     case MIDORI_IDENT_SAFARI:
         return g_strdup_printf ("Mozilla/5.0 (%s; U; %s; %s) "
-            "AppleWebKit/532+ (KHTML, like Gecko) Safari/419.3 %s",
-                                platform, os, lang, appname);
+            "AppleWebKit/532+ (KHTML, like Gecko) Safari/%s %s",
+                                platform, os, lang, webcore, appname);
     case MIDORI_IDENT_FIREFOX:
         return g_strdup_printf ("Mozilla/5.0 (%s; U; %s; %s; rv:1.8.1) "
             "Gecko/20061010 Firefox/2.0 %s",
@@ -1174,6 +1201,9 @@ midori_web_settings_set_property (GObject*      object,
     case PROP_COMPACT_SIDEPANEL:
         web_settings->compact_sidepanel = g_value_get_boolean (value);
         break;
+    case PROP_SHOW_PANEL_CONTROLS:
+        web_settings->show_panel_controls = g_value_get_boolean (value);
+        break;
     case PROP_RIGHT_ALIGN_SIDEPANEL:
         web_settings->right_align_sidepanel = g_value_get_boolean (value);
         break;
@@ -1301,12 +1331,20 @@ midori_web_settings_set_property (GObject*      object,
         {
             gchar* string = generate_ident_string (web_settings->identify_as);
             katze_assign (web_settings->ident_string, string);
+            #if WEBKIT_CHECK_VERSION (1, 1, 11)
+            g_object_set (web_settings, "user-agent", string, NULL);
+            #endif
             g_object_notify (object, "ident-string");
         }
         break;
     case PROP_IDENT_STRING:
         if (web_settings->identify_as == MIDORI_IDENT_CUSTOM)
+        {
             katze_assign (web_settings->ident_string, g_value_dup_string (value));
+            #if WEBKIT_CHECK_VERSION (1, 1, 11)
+            g_object_set (web_settings, "user-agent", web_settings->ident_string, NULL);
+            #endif
+        }
         break;
     case PROP_CACHE_SIZE:
         web_settings->cache_size = g_value_get_int (value);
@@ -1386,6 +1424,9 @@ midori_web_settings_get_property (GObject*    object,
     case PROP_COMPACT_SIDEPANEL:
         g_value_set_boolean (value, web_settings->compact_sidepanel);
         break;
+    case PROP_SHOW_PANEL_CONTROLS:
+        g_value_set_boolean (value, web_settings->show_panel_controls);
+        break;
     case PROP_RIGHT_ALIGN_SIDEPANEL:
         g_value_set_boolean (value, web_settings->right_align_sidepanel);
         break;
