@@ -16,6 +16,7 @@
 #include "midori-stock.h"
 #include "midori-view.h"
 #include "midori-viewable.h"
+#include "midori-bookmark-store.h"
 
 #include "sokoke.h"
 
@@ -725,8 +726,10 @@ midori_bookmarks_open_in_window_activate_cb (GtkWidget*       menuitem,
 
     if (uri && *uri)
     {
-        MidoriBrowser* browser = midori_browser_get_for_widget (GTK_WIDGET (bookmarks));
-        g_signal_emit_by_name (browser, "new-window", uri);
+        MidoriBrowser* new_browser = midori_app_create_browser (bookmarks->app);
+        midori_app_add_browser (bookmarks->app, new_browser);
+        gtk_widget_show (GTK_WIDGET (new_browser));
+        midori_browser_add_uri (new_browser, uri);
     }
 }
 
@@ -892,7 +895,7 @@ midori_bookmarks_init (MidoriBookmarks* bookmarks)
     bookmarks->net = katze_net_new ();
 
     /* Create the treeview */
-    model = gtk_tree_store_new (1, KATZE_TYPE_ITEM);
+    model = midori_bookmark_store_new (1, KATZE_TYPE_ITEM);
     treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
     gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);
     column = gtk_tree_view_column_new ();
@@ -932,6 +935,7 @@ midori_bookmarks_finalize (GObject* object)
 {
     MidoriBookmarks* bookmarks = MIDORI_BOOKMARKS (object);
 
+    midori_bookmarks_disconnect_folder (bookmarks, bookmarks->array);
     if (bookmarks->app)
         g_object_unref (bookmarks->app);
     g_object_unref (bookmarks->net);
